--
-- $Id: mkhref.hs,v 1.20 2006/05/31 15:20:19 aamine Exp $
--
-- Copyright (c) 2003-2006 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module Main (main) where

import HrefDatabase
import PathUtils
import Data.Char
import System.IO
import System.Environment

main :: IO ()
main = do prefix <- hrefDatabasePrefix
          ents <- return . concat =<< mapM loadModule =<< getArgs
          mapM_ (putEntry prefix) ents
          putIndex prefix ents

loadModule :: FilePath -> IO [Entry]
loadModule path = return . parseModule (basename path) =<< readFile path

parseModule :: String -> String -> [Entry]
parseModule mod = map (chunkToEntry mod) . chunks

chunkToEntry :: String -> String -> Entry
chunkToEntry mod cs = Entry (EntrySpec mod (functionName cs)) cs
  where
    functionName = unparenthesize . funcnameFromChunk
    
    funcnameFromChunk = head . words . head . lines

    unparenthesize ('(':s) = init s
    unparenthesize w = w

chunks :: String -> [String]
chunks = parseLines . lines
  where
    parseLines [] = []
    parseLines xs = let (header, cont) = break isBody xs
                        (body, xs') = span isBody cont
                    in
                        unlines (header ++ body) : parseLines xs'

    isBody ""     = True
    isBody (c:cs) = isSpace c
